package engine;

import java.lang.Thread;
import java.util.*;
import java.lang.*;
import ui.*;
import functions.Settings;
import java.net.*;


public class Task extends Thread
{
	private Plugin mPlugin;
	private Vector<String> mTableRow;
	private volatile TaskState mState;
	private TaskPriority mPriority;
	private int mLastDone;
	private long mLastTime;
	private int mLastEta[];
	private boolean mStarted;

	private static String mTaskStates[] = { "Pobieranie", "W kolejce", "Oczekuje", "Zakończone", "Zatrzymane", "Błąd" };
	private static String mTaskPriorities[] = { "Najniższy", "Niski", "Normalny", "Wysoki", "Na wczoraj" };
	private static int mActiveTasks;
	private static Vector<Task> mTasks = new Vector<Task>();
	private static Vector<Vector<String>> mTableRows = new Vector<Vector<String>>();
	private static Manager mPluginManager;

	public Task(Plugin plugin, String link, String dest, TaskPriority priority)
	{
		mPlugin = plugin;
		mPriority = priority;
		mTableRow = new Vector<String>();
		mState = TaskState.tsStopped;

		mTableRow.add(plugin.getIcon()); /* Ikonka */
		mTableRow.add(link); /* Link */
		mTableRow.add("0");  /* Postep */
		mTableRow.add(mTaskStates[mState.ordinal()]); /* Status */
		mTableRow.add(mTaskPriorities[mPriority.ordinal()]); /* Priorytet */
		mTableRow.add(""); /* Rozmiar */
		mTableRow.add(dest); /* Docelowy */
		mTableRow.add(""); /* ETA */

		mPlugin.initTask(this);

		mLastEta = new int[6];
		mStarted = false;
	}

	public Plugin getTaskPlugin()
	{
		return mPlugin;
	}

	public TaskState getTaskState()
	{
		return mState;
	}

	public String getLink()
	{
		return mTableRow.elementAt(1);
	}

	public String getDestDir()
	{
		return mTableRow.elementAt(6);
	}

	public void setProgress(int done, int total)
	{
		mTableRow.setElementAt(Integer.toString((done * 100) / total), 2);
		if (total < 1024)
			mTableRow.setElementAt(Integer.toString(total) + "B", 5);
		else if (total < 1024 * 1024)
			mTableRow.setElementAt(Integer.toString(total / 1024) + "KiB", 5);
		else
			mTableRow.setElementAt(Integer.toString(total / 1024 / 1024) + "MiB", 5);

		/* Liczymy ETA */
		if (mLastTime > 0)
		{
			long time = System.currentTimeMillis() - mLastTime;
			int diff = done - mLastDone;
			int eta = (int)((time * (total - done) / diff) / 1000);

			mLastEta[5] = mLastEta[4];
			mLastEta[4] = mLastEta[3];
			mLastEta[2] = mLastEta[1];
			mLastEta[1] = mLastEta[0];
			mLastEta[0] = eta;

			setETA((mLastEta[0] + mLastEta[1] + mLastEta[2] + mLastEta[3] + mLastEta[4] + mLastEta[5]) / 6);
		}	

		mLastDone = done;
		mLastTime = System.currentTimeMillis();
	}

	public String toString()
	{
		return mTableRow.elementAt(1) + "|" + mState.ordinal() + "|" + mPriority.ordinal() + "|" + mTableRow.elementAt(6);
	}

	public Vector<String> getTableRow()
	{
		return mTableRow;
	}

	public synchronized void setState(TaskState newState)
	{
		TaskState oldState = mState;

		if ((mState == newState) || (mState == TaskState.tsDone))
			return ;

		mState = newState;
		mTableRow.setElementAt(mTaskStates[mState.ordinal()], 3);

		/* Uruchamiamy zadanie od zera */
		if (newState == TaskState.tsRunning)
		{
			if (mStarted)
				notify();
			else
			{
				start();
				mStarted = true;
			}
		}
	}

	public void setState(TaskState newState, String descr)
	{
		setState(newState);
		mTableRow.setElementAt(mTaskStates[mState.ordinal()] + " (" + descr + ")", 3);		
	}

	public void setETA(int sec)
	{
		String eta = "";
		int m = (sec / 60) % 60;
		int h = (sec / 60 / 60);
		sec = sec % 60;

		if (h > 0)
			eta += new Integer(h).toString() + "g ";
		if (m > 0)
			eta += new Integer(m).toString() + "m ";
		eta += new Integer(sec).toString() + "s";
		mTableRow.setElementAt(eta, 7);
	}

	public void run()
	{
		while (true)
		{
			mPlugin.run(this);
			/* Jezeli jestesmy tutaj to plik sie pobral albo blad */
			mActiveTasks--;
			/* Odpalamy kolejne zadanie */
			startNextTask();
			/* Aktualny watek czeka na obudzenie */
			if ((mState != TaskState.tsDone) && (mState != TaskState.tsError))
			{
				try
				{
					synchronized(this)
					{
						while(mState == TaskState.tsStopped)
							wait();
					}
					/* Wznowiony */
				}
				catch (Exception e)	
				{
					break;
				}
			}
		}
	}

	public static void setPluginManager(Manager m)
	{
		mPluginManager = m;
	}

	public static Vector<Vector<String>> getTableRows()
	{
		return mTableRows;
	}

	public static void addTask(String link, TaskState s, TaskPriority p, String destDir)
	{
		for(Vector<String> vect:mTableRows) 
		{
			if(link.equals(vect.elementAt(1))) 
				return;
		}
				
		/* Sprawdzamy czy mamy jakis plugin co da rade temu linkowi */
		Plugin plugin = mPluginManager.checkLink(link);
		if (plugin == null)
			return;
				
		Task task = new Task(plugin, link, destDir, p);
		mTasks.add(task);
		mTableRows.add(task.getTableRow());
		task.setState(s);
	}

	public static void startTask(int id)
	{
		if ((id < 0) || (id >= mTasks.size()))
			return;

		if ((mActiveTasks < Integer.parseInt(Settings.getValue("MAX_CONNECTIONS"))) &&
		    (mTasks.elementAt(id).getTaskPlugin().allowMoreTasks()))
		{
			mTasks.elementAt(id).setState(TaskState.tsRunning);			
			mActiveTasks++;
			Thread.yield();			
		}
		else
			mTasks.elementAt(id).setState(TaskState.tsQueue);
	}

	public static void pauseTask(int id)
	{
		if ((id < 0) || (id >= mTasks.size()))
			return;

		mTasks.elementAt(id).setState(TaskState.tsStopped);
	}

	public static void removeTask(int id)
	{
		pauseTask(id);
		mTableRows.remove(id);
		mTasks.remove(id);
	}

	public static void startNextTask()
	{
		/* Znajdz nastepne zadanie do odpalenia */
		for (Task task:mTasks)
		{
			if ((task.getTaskState() == TaskState.tsQueue) && (task.getTaskPlugin().allowMoreTasks()))
			{
				task.setState(TaskState.tsRunning);
				mActiveTasks++;
				Thread.yield();
			}

			if (mActiveTasks >= Integer.parseInt(Settings.getValue("MAX_CONNECTIONS")))
				return;
		}
	}

	public static String baseName(String path)
	{
		for(int i = path.length() - 1;i>=0;i--)
		{
			if (path.charAt(i) == '/')
				return path.substring(i + 1);
		}
		return path;
	}

	public static URLConnection makeConnection(String link)
	{
		URLConnection c = null;
		int tries = 0;
		while(tries < Integer.parseInt(Settings.getValue("MAX_TRY")))
		{
			try
			{
				URL url = new URL(link);			
				if (Settings.getValue("PROXY").equals("yes"))
				{
					String tmp[] = Settings.getValue("PROXY_SERVER").split(":");
					c = url.openConnection(new Proxy(Proxy.Type.HTTP, new InetSocketAddress(tmp[0], Integer.parseInt(tmp[1]))));
				}
				else
				{
					c = url.openConnection();
				}

				//c.setConnectTimeout(Integer.parseInt(Settings.getValue("TRY_TIME")));
			
				break;
			}
			catch (Exception e)
			{
				c = null;
			}

			try
			{
				Thread.sleep(500);
			}
			catch (Exception e)
			{
				
			}
			tries++;
		}
		return c;
	}

	public static void saveLinks()
	{
		TaskFile.saveTo(mTasks);
	}

	public static void loadLinks()
	{
		TaskFile.loadFrom();
	}

	public static void setPluginEnabled(String name, Boolean enabled)
	{		
		if (enabled)
			mPluginManager.enablePlugin(name);
		else
			mPluginManager.disablePlugin(name);
	}

	public static Vector<Vector<Object>> getPlugins()
	{
		return mPluginManager.getPlugins();
	}
}
